package com.exquisiteapplicationsolutions.tis.activities;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.location.Address;
import android.location.Geocoder;
import android.location.Location;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.maps.CameraUpdate;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.MapFragment;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.LatLngBounds;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;
import com.exquisiteapplicationsolutions.tis.CustomMarker;
import com.exquisiteapplicationsolutions.tis.DBParameters;
import com.exquisiteapplicationsolutions.tis.LatLngInterpolator;
import com.exquisiteapplicationsolutions.tis.LatLngInterpolator.LinearFixed;
import com.exquisiteapplicationsolutions.tis.MarkerAnimation;
import com.exquisiteapplicationsolutions.tis.R;
import com.exquisiteapplicationsolutions.tis.TISDBHelper;
import com.exquisiteapplicationsolutions.tis.objects.Jobs;

import org.json.JSONArray;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map.Entry;

public class JobMapActivity extends AppCompatActivity {

    private static final String sDBName = "TIS.db";
    private static final int iVersionVal = 8;
    private GoogleMap googleMap;
    private HashMap markersHashMap;
    private Iterator<Entry> iter;
    private CameraUpdate cu;
    private CustomMarker customMarkerOne, customMarkerTwo, customMarkerThree;
    private static ArrayList<ArrayList<String>> jobArray;
    private String sJobID;
    private String sUserID = "";
    private Double puLat = 0.0;
    private Double puLong = 0.0;
    private Double delLat = 0.0;
    private Double delLong = 0.0;
    private String sPUAddress;
    private String sDELAddress;
    private Jobs jobObj;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_job_map);

        SharedPreferences prefs = getSharedPreferences("TISSharedPrefs",
                Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = prefs.edit();
        String sLoggedInUserCompany = prefs.getString("LoggedInUserCompany", "No Value");
        sUserID = prefs.getString("UserID", "No Value");
        TextView otvUserName = (TextView) findViewById(R.id.tvUserName);
        otvUserName.setText(sLoggedInUserCompany.substring(0,sLoggedInUserCompany.indexOf('-')));

        try {
            // get passed in values
            sJobID = getIntent().getExtras().getString("JobID");

            // get info from job for map
            getAddress();

            // Loading map
            initilizeMap();
            initializeUiSettings();
            initializeMapLocationSettings();
            initializeMapTraffic();
            initializeMapType();
            initializeMapViewSettings();

        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    @Override
    protected void onResume() {
        super.onResume();
        // initilizeMap();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_job_map, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_icon_meanings) {
            Intent i = new Intent(getApplicationContext(), HelpActivity.class);
            i.putExtra("ScreenID", "IconMeanings");
            startActivity(i);
            return true;
        }
        if (id == R.id.action_help) {
            return true;
        }
        if (id == R.id.action_about) {
            Intent i = new Intent(getApplicationContext(), AboutActivity.class);
            startActivity(i);
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    private void initilizeMap() {

        googleMap = ((MapFragment) getFragmentManager().findFragmentById(R.id.mapFragment)).getMap();

        // check if map is created successfully or not
        if (googleMap == null) {
            Toast.makeText(getApplicationContext(), "Sorry! unable to create maps", Toast.LENGTH_SHORT).show();
        }
        googleMap.setMyLocationEnabled(true);

        (findViewById(R.id.mapFragment)).getViewTreeObserver().addOnGlobalLayoutListener(
                new android.view.ViewTreeObserver.OnGlobalLayoutListener() {

                    @Override
                    public void onGlobalLayout() {
                        // gets called after layout has been done but before
                        // display
                        // so we can get the height then hide the view
                        if (android.os.Build.VERSION.SDK_INT >= 16) {
                            (findViewById(R.id.mapFragment)).getViewTreeObserver().removeOnGlobalLayoutListener(this);
                        } else {
                            (findViewById(R.id.mapFragment)).getViewTreeObserver().removeGlobalOnLayoutListener(this);
                        }
                        setCustomMarkerOnePosition();
                        setCustomMarkerTwoPosition();
                        setCustomMarkerThreePostion();
                        zoomToMarkers(findViewById(R.id.mapFragment));
                        // plotMarkers(markerList);
                    }
                });
    }

    void setCustomMarkerOnePosition() {
        customMarkerOne = new CustomMarker("Pickup", puLat, puLong, sPUAddress, 1);
        addMarker(customMarkerOne);
    }

    void setCustomMarkerTwoPosition() {
        customMarkerTwo = new CustomMarker("Delivery", delLat, delLong, sDELAddress, 2);
        addMarker(customMarkerTwo);
    }

    void setCustomMarkerThreePostion() {
        try {
            googleMap.setMyLocationEnabled(true);
            Location myLocation = googleMap.getMyLocation();
            customMarkerThree = new CustomMarker("Driver", myLocation.getLatitude(), myLocation.getLongitude(), "My Location", 3);
            addMarker(customMarkerThree);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void startAnimation(View v) {
        animateMarker(customMarkerOne, new LatLng(40.0675716, 40.7297251));
    }

    public void zoomToMarkers(View v) {
        zoomAnimateLevelToFitMarkers(120);
    }

    public void animateBack(View v) {
        animateMarker(customMarkerOne, new LatLng(32.0675716, 27.7297251));
    }

    public void initializeUiSettings() {
        googleMap.getUiSettings().setCompassEnabled(true);
        googleMap.getUiSettings().setRotateGesturesEnabled(false);
        googleMap.getUiSettings().setTiltGesturesEnabled(true);
        googleMap.getUiSettings().setZoomControlsEnabled(true);
        googleMap.getUiSettings().setMyLocationButtonEnabled(true);
    }

    public void initializeMapLocationSettings() {
        googleMap.setMyLocationEnabled(true);
    }

    public void initializeMapTraffic() {
        googleMap.setTrafficEnabled(true);
    }

    public void initializeMapType() {
        googleMap.setMapType(GoogleMap.MAP_TYPE_NORMAL);
    }

    public void initializeMapViewSettings() {
        googleMap.setIndoorEnabled(true);
        googleMap.setBuildingsEnabled(false);
    }

    // this is method to help us set up a Marker that stores the Markers we want
    // to plot on the map
    public void setUpMarkersHashMap() {
        if (markersHashMap == null) {
            markersHashMap = new HashMap<CustomMarker, Marker>();
        }
    }

    // this is method to help us add a Marker into the hashmap that stores the
    // Markers
    public void addMarkerToHashMap(CustomMarker customMarker, Marker marker) {
        setUpMarkersHashMap();
        markersHashMap.put(customMarker, marker);
    }

    // this is method to help us find a Marker that is stored into the hashmap
    public Marker findMarker(CustomMarker customMarker) {
        iter = markersHashMap.entrySet().iterator();
        while (iter.hasNext()) {
            Entry mEntry = (Entry) iter.next();
            CustomMarker key = (CustomMarker) mEntry.getKey();
            if (customMarker.getCustomMarkerId().equals(key.getCustomMarkerId())) {
                Marker value = (Marker) mEntry.getValue();
                return value;
            }
        }
        return null;
    }

    // this is method to help us add a Marker to the map
    public void addMarker(CustomMarker customMarker) {
        MarkerOptions markerOption = null;
        switch (customMarker.getIcon()) {
            case 1:
                markerOption = new MarkerOptions().position(
                        new LatLng(customMarker.getCustomMarkerLatitude(), customMarker.getCustomMarkerLongitude()))
                        .title(customMarker.getTitle())
                        .icon(BitmapDescriptorFactory.fromResource(R.drawable.customs));
                //.icon(BitmapDescriptorFactory.defaultMarker());
                break;
            case 2:
                markerOption = new MarkerOptions().position(
                        new LatLng(customMarker.getCustomMarkerLatitude(), customMarker.getCustomMarkerLongitude()))
                        .title(customMarker.getTitle())
                        .icon(BitmapDescriptorFactory.fromResource(R.drawable.office_building));
                break;
            case 3:
                markerOption = new MarkerOptions().position(
                        new LatLng(customMarker.getCustomMarkerLatitude(), customMarker.getCustomMarkerLongitude()))
                        .title(customMarker.getTitle())
                        .icon(BitmapDescriptorFactory.fromResource(R.drawable.truck3));
                break;
        }

        Marker newMark = googleMap.addMarker(markerOption);
        addMarkerToHashMap(customMarker, newMark);
    }

    // this is method to help us remove a Marker
    public void removeMarker(CustomMarker customMarker) {
        if (markersHashMap != null) {
            if (findMarker(customMarker) != null) {
                findMarker(customMarker).remove();
                markersHashMap.remove(customMarker);
            }
        }
    }

    // this is method to help us fit the Markers into specific bounds for camera
    // position
    public void zoomAnimateLevelToFitMarkers(int padding) {
        LatLngBounds.Builder b = new LatLngBounds.Builder();
        iter = markersHashMap.entrySet().iterator();

        while (iter.hasNext()) {
            Entry mEntry = (Entry) iter.next();
            CustomMarker key = (CustomMarker) mEntry.getKey();
            LatLng ll = new LatLng(key.getCustomMarkerLatitude(), key.getCustomMarkerLongitude());
            b.include(ll);
        }
        LatLngBounds bounds = b.build();

        // Change the padding as per needed
        cu = CameraUpdateFactory.newLatLngBounds(bounds, padding);
        googleMap.animateCamera(cu);
    }

    // this is method to help us move a Marker.
    public void moveMarker(CustomMarker customMarker, LatLng latlng) {
        if (findMarker(customMarker) != null) {
            findMarker(customMarker).setPosition(latlng);
            customMarker.setCustomMarkerLatitude(latlng.latitude);
            customMarker.setCustomMarkerLongitude(latlng.longitude);
        }
    }

    // this is method to animate the Marker. There are flavours for all Android
    // versions
    public void animateMarker(CustomMarker customMarker, LatLng latlng) {
        if (findMarker(customMarker) != null) {

            LatLngInterpolator latlonInter = new LinearFixed();
            latlonInter.interpolate(20,
                    new LatLng(customMarker.getCustomMarkerLatitude(), customMarker.getCustomMarkerLongitude()), latlng);

            customMarker.setCustomMarkerLatitude(latlng.latitude);
            customMarker.setCustomMarkerLongitude(latlng.longitude);

            if (android.os.Build.VERSION.SDK_INT >= 14) {
                MarkerAnimation.animateMarkerToICS(findMarker(customMarker), new LatLng(customMarker.getCustomMarkerLatitude(),
                        customMarker.getCustomMarkerLongitude()), latlonInter);
            } else if (android.os.Build.VERSION.SDK_INT >= 11) {
                MarkerAnimation.animateMarkerToHC(findMarker(customMarker), new LatLng(customMarker.getCustomMarkerLatitude(),
                        customMarker.getCustomMarkerLongitude()), latlonInter);
            } else {
                MarkerAnimation.animateMarkerToGB(findMarker(customMarker), new LatLng(customMarker.getCustomMarkerLatitude(),
                        customMarker.getCustomMarkerLongitude()), latlonInter);
            }
        }
    }

    private void getAddress() {
        // need to get both the pickup and delivery addresses to get the lat and long for both to put on the map
        TISDBHelper oDB = new TISDBHelper(getApplicationContext(), sDBName,
                null, iVersionVal);
        try {
            oDB.createDataBase();
        } catch (IOException ioe) {
            throw new Error("Unable to create database");
        }

        oDB.openDataBase();
        String sSQL = "Select *   ";
        sSQL += "from jobs ";
        sSQL += "where id = " + sJobID;

        Log.d("sSQL", sSQL);

        jobArray = oDB.selectRecordsFromDBList(sSQL, null);

        // closes the database
        try {
            oDB.close();
        } catch (Throwable e) {
            e.printStackTrace();
        }

        ArrayList<Jobs> allJobs = new ArrayList<Jobs>();
        Integer iSize = jobArray.size();
        Log.d("iSize", iSize.toString());
        jobObj = new Jobs();
        if (iSize > 0) {
            for (ArrayList<String> sValue : jobArray) {
                jobObj.job_id = sValue.get(1);
                jobObj.job_status_id = sValue.get(3);
                jobObj.date_time = sValue.get(2);
                jobObj.status_name = sValue.get(4);
                jobObj.assigned_id = sValue.get(5);
                jobObj.importer_id = sValue.get(19);
                jobObj.importer_name = sValue.get(20);
                jobObj.manifest_num = sValue.get(6);
                jobObj.container_num = sValue.get(7);
                jobObj.delivery_address_line1 = sValue.get(8);
                jobObj.delivery_address_line2 = sValue.get(9);
                jobObj.delivery_city = sValue.get(10);
                jobObj.delivery_state = sValue.get(11);
                jobObj.delivery_postal_code = sValue.get(12);
                jobObj.pickup_address_line1 = sValue.get(13);
                jobObj.pickup_address_line2 = sValue.get(14);
                jobObj.pickup_city = sValue.get(15);
                jobObj.pickup_state = sValue.get(16);
                jobObj.pickup_postal_code = sValue.get(17);
                jobObj.username = sValue.get(18);
                jobObj.job_num = sValue.get(21);
                allJobs.add(jobObj);
            }
        }

        int iCalcWay = 1; // 1=MapQuest, 2=Google

        // get for pickup address
        sPUAddress = jobObj.pickup_address_line1 + " " + jobObj.pickup_address_line2 + "," + jobObj.pickup_city + "," + jobObj.pickup_state + "," + jobObj.pickup_postal_code;
        convertAddress(sPUAddress,1,iCalcWay);

        // get for delivery address
        sDELAddress = jobObj.delivery_address_line1 + " " + jobObj.delivery_address_line2 + "+" + jobObj.delivery_city + "," + jobObj.delivery_state + "," + jobObj.delivery_postal_code;
        convertAddress(sDELAddress,2,iCalcWay);
    }


    public void convertAddress(String sAddress, Integer iType, Integer iCalcWay) {

        if (iCalcWay == 1) {
        /* MapQuest Way */
            Boolean bRtnVal = false;

            DBParameters oDBParameters = new DBParameters();

            // WebServer Request URL
            String serverURL = "http://www.mapquestapi.com/geocoding/v1/address?key=axvWuNxhlhdVvnwBzUvHvrAFufAQNacG";

            String data = "";

            //&location="+sAddress;

            try {
                // Set Request parameter

                data += "&" + URLEncoder.encode("location", "UTF-8") + "=" + sAddress;
            //    data += URLEncoder.encode("sAddress", "UTF-8");
                } catch (UnsupportedEncodingException e) {
                    e.printStackTrace();
                }

            BufferedReader reader = null;
            StringBuilder sb = new StringBuilder();
            // Send data
            try {

                // Defined URL  where to send data
                URL url = new URL(serverURL);

                // Send POST data request
                URLConnection conn = url.openConnection();
                conn.setDoOutput(true);
                OutputStreamWriter wr = new OutputStreamWriter(conn.getOutputStream());
                wr.write(data);
                wr.flush();

                // Get the server response
                reader = new BufferedReader(new InputStreamReader(conn.getInputStream()));
                String line = null;

                // Read Server Response
                while ((line = reader.readLine()) != null) {
                    // Append server response in string
                    sb.append(line + "");

                }
            } catch (Exception ex) {
                ex.printStackTrace();
                Log.e("convertAddress Error:", ex.toString());
            } finally {
                try {
                    reader.close();
                } catch (Exception ex) {
                }
            }

            try {
                JSONObject oJSONObject = new JSONObject(sb.toString());
                JSONArray oJSONResults = new JSONArray(oJSONObject.getString("results"));
                JSONObject oResults = oJSONResults.getJSONObject(0);
                JSONArray oJSONLocations = new JSONArray(oResults.getString("locations"));
                JSONObject oLocations = oJSONLocations.getJSONObject(0);
                JSONObject oLatLng = oLocations.getJSONObject("latLng");

                double lat = Double.parseDouble(oLatLng.getString("lat"));
                double lng = Double.parseDouble(oLatLng.getString("lng"));

                if (iType == 1) {
                    puLat = lat;
                    puLong = lng;
                } else {
                    delLat = lat;
                    delLong = lng;
                }

            } catch (Exception ex) {
                Log.d("getMessages():", ex.toString());
            }

        } else {
        /* Google Way */
            Geocoder geoCoder = new Geocoder(this);
            try {
                if (sAddress != null && !sAddress.isEmpty()) {
                    try {
                        List<Address> addressList = geoCoder.getFromLocationName(sAddress, 1);
                        if (addressList != null && addressList.size() > 0) {
                            double lat = addressList.get(0).getLatitude();
                            double lng = addressList.get(0).getLongitude();
                            if (iType == 1) {
                                puLat = lat;
                                puLong = lng;
                            } else {
                                delLat = lat;
                                delLong = lng;
                            }
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                    } // end catch
                } // end if
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
    } // end convert address
}



